using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace ReFrameworker
{
    using System;
    using System.IO;

    internal class CodeChanger
    
    {
        string binaryName;
        string disassembledBinaryName;

        public CodeChanger(string BinaryName)
        {
            this.binaryName = BinaryName;
            disassembledBinaryName = binaryName + Configuration.disassembledExtension;
        }

        public void Disassemble()
        {
            FileUtilities.Disassemble(binaryName);
        }

        public void Assemble()
        {
            FileUtilities.Assemble(binaryName);
        }
        
        private void HandleCodeBeforeInjection(StreamReader file, string searchMethodName,  StreamWriter fileout)
        {
            string str = null;
            try
            {
                //read until we reach the method name
                while (((str = file.ReadLine()) != null) && !str.Contains(searchMethodName))
                {
                    fileout.WriteLine(str);
                }
                if (str == null)
                    throw new Exception("payload injection point location not found");

                //now we are at the method name
                //write the method name line
                fileout.WriteLine(str);

                
                  while (((str = file.ReadLine()) != null) && !str.Contains("{"))
                {
                    fileout.WriteLine(str);
                }
                //now we are at the beginning of the method
                //write the { character
                fileout.WriteLine(str);                       
            }
            catch (Exception e)
            {
                throw new Exception("Error while handling IL code (" + e.Message + ")");
            }
        }

        private void HandleCodeAfterInjection(StreamReader file, StreamWriter fileout)
        {
            string str = null;
            try
            {
                //Continue to read until end of file
                while ((str = file.ReadLine()) != null)
                {
                    fileout.WriteLine(str);
                }
                //now we are at the end of the file
            }
            catch (Exception e)
            {
                throw new Exception("Error while handling MSIL code after target method location (" + e.Message + ")");
            }
        }
        
        public void InjectExternalRef(ReferenceModule payloadExternalRef)
        {
            string tempFileName = disassembledBinaryName + Configuration.tempExtension;
            StreamReader reader = new StreamReader(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            StreamReader reader2 = new StreamReader(Configuration.RefsDir + "\\" + payloadExternalRef.InjectedRefFile);
            StreamWriter writer = new StreamWriter(Configuration.DisassembledDir + "\\" + tempFileName);
            string str2 = null;
            string str3 = ".assembly";
            try
            {
                //Read the first line of text
                //Continue to read until we reach the method name
                string str4;
                while (((str2 = reader.ReadLine()) != null) && !str2.Contains(str3))
                {
                    writer.WriteLine(str2);
                }
                //now we are at the first external ref
                //write the ref file
                while ((str4 = reader2.ReadLine()) != null)
                {
                    writer.WriteLine(str4);
                }
                //write the first line of external (from before)
                writer.WriteLine(str2);
                //Continue to read until we reach the EOF
                while ((str2 = reader.ReadLine()) != null)
                {
                    writer.WriteLine(str2);
                }

                //releasing resources and removing temp files
                reader.Close();
                reader2.Close();
                writer.Close();
                File.Delete(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                File.Move(Configuration.DisassembledDir + "\\" + tempFileName, Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            }
            catch (Exception e)
            {
                throw new Exception("Error while injecting the reference "+payloadExternalRef.InjectedRefFile  +"(" + e.Message + ")");
            }
            
        }

        public void InjectMethod(MethodModule payloadMethod)
        {
            string tempFileName = disassembledBinaryName + Configuration.tempExtension;
            StreamReader reader = new StreamReader(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            StreamReader reader2 = new StreamReader(Configuration.MethodsDir + "\\" + payloadMethod.MethodCodeFile);
            StreamWriter writer = new StreamWriter(Configuration.DisassembledDir + "\\" + tempFileName);
            string str2 = null;
            string injectionLocation = payloadMethod.InjectionLocation;
            try
            {
                //read until we reach the Injection Location
                string str4;
                while (((str2 = reader.ReadLine()) != null) && !str2.Contains(injectionLocation))
                {
                    writer.WriteLine(str2);
                }
                if (payloadMethod.BeforeLocation)
                {
                    //write the method
                    while ((str4 = reader2.ReadLine()) != null)
                    {
                        //Read the next line
                        writer.WriteLine(str4);
                    }
                    //write the line (from before)
                    writer.WriteLine(str2);
                }
                else
                {
                    //write the line (from before)
                    writer.WriteLine(str2);
                    //write the method
                    while ((str4 = reader2.ReadLine()) != null)
                    {
                        writer.WriteLine(str4);
                    }
                }
                while ((str2 = reader.ReadLine()) != null)
                //Continue to read until we reach the EOF
                {
                    writer.WriteLine(str2);
                }
                //releasing resources and removing temp files
                reader.Close();
                reader2.Close();
                writer.Close();
                File.Delete(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                File.Move(Configuration.DisassembledDir + "\\" + tempFileName, Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            }
            catch (Exception e)
            {
                throw new Exception("Error while injecting the method " + payloadMethod.MethodCodeFile + "(" + e.Message + ")");
            }
            
        }


        public void InjectClass(ClassModule payloadClass)
        {
            string tempFileName = disassembledBinaryName + Configuration.tempExtension;
            StreamReader reader = new StreamReader(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            StreamReader reader2 = new StreamReader(Configuration.ClassesDir + "\\" + payloadClass.ClassCodeFile);
            StreamWriter writer = new StreamWriter(Configuration.DisassembledDir + "\\" + tempFileName);
            string str2 = null;
            string injectionLocation = payloadClass.InjectionLocation;
            try
            {
                //read until we reach the Injection Location
                string str4;
                while (((str2 = reader.ReadLine()) != null) && !str2.Contains(injectionLocation))
                {
                    writer.WriteLine(str2);
                }
                if (payloadClass.BeforeLocation)
                {
                    //write the method
                    while ((str4 = reader2.ReadLine()) != null)
                    {
                        //Read the next line
                        writer.WriteLine(str4);
                    }
                    //write the line (from before)
                    writer.WriteLine(str2);
                }
                else
                {
                    //write the line (from before)
                    writer.WriteLine(str2);
                    //write the method
                    while ((str4 = reader2.ReadLine()) != null)
                    {
                        writer.WriteLine(str4);
                    }
                }
                while ((str2 = reader.ReadLine()) != null)
                //Continue to read until we reach the EOF
                {
                    writer.WriteLine(str2);
                }
                //releasing resources and removing temp files
                reader.Close();
                reader2.Close();
                writer.Close();
                File.Delete(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                File.Move(Configuration.DisassembledDir + "\\" + tempFileName, Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            }
            catch (Exception e)
            {
                throw new Exception("Error while injecting the class " + payloadClass.ClassCodeFile + "(" + e.Message + ")");
            }

        }



        public void InjectPayloadCode(PayloadModule payloadInjectedCode, PayloadModule.InjectionModeEnum injectionMode)
        {
            try
            {
                //Pass the file path and file name to the StreamReader constructor
                string tempFileName = disassembledBinaryName + Configuration.tempExtension;
                StreamReader file = new StreamReader(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                StreamReader reader2 = new StreamReader(Configuration.PayloadsDir + "\\" + payloadInjectedCode.InjectedCodeFile);
                StreamWriter fileout = new StreamWriter(Configuration.DisassembledDir + "\\" + tempFileName);
                this.HandleCodeBeforeInjection(file, payloadInjectedCode.InjectionLocation, fileout);
                if (injectionMode == PayloadModule.InjectionModeEnum.PostAppend)
                    this.MergeCodePost(file, reader2, fileout, payloadInjectedCode.StackSize, payloadInjectedCode.ConsiderLineNumbering);
                else
                    this.MergeCodePre(file, reader2, fileout, payloadInjectedCode.StackSize,payloadInjectedCode.ConsiderLineNumbering);
                this.HandleCodeAfterInjection(file, fileout);
                
                //releasing resources and removing temp files
                file.Close();
                reader2.Close();
                fileout.Close();
                File.Delete(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                File.Move(Configuration.DisassembledDir + "\\" + tempFileName, Configuration.DisassembledDir + "\\" + disassembledBinaryName);
            }
            catch (Exception e)
            {
                throw new Exception("Error while injecting the payload " + payloadInjectedCode.InjectedCodeFile + "(" + e.Message + ")");
            }
        }

        public void ReplaceWithPayloadCode(PayloadModule payloadInjectedCode)
        {                     
            string str = null;
            string searchMethodName = payloadInjectedCode.InjectionLocation;
            try
            {
                //Pass the file path and file name to the StreamReader constructor
                string tempFileName = disassembledBinaryName + Configuration.tempExtension;
                StreamReader originalFile = new StreamReader(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                StreamReader payloadFile = new StreamReader(Configuration.PayloadsDir + "\\" + payloadInjectedCode.InjectedCodeFile);
                StreamWriter modifiedFile = new StreamWriter(Configuration.DisassembledDir + "\\" + tempFileName);

                //read until we reach the method name
                while (((str = originalFile.ReadLine()) != null) && !str.Contains(searchMethodName))
                {

                    modifiedFile.WriteLine(str);
                }

                //write the method name line
                modifiedFile.WriteLine(str);
                
                //look for the beginning of code - looking for the { character
                while (((str = originalFile.ReadLine()) != null) && !str.Contains("{"))
                {

                    modifiedFile.WriteLine(str);
                }

                //now we are at the beginning of the method code
                //write the { character
                modifiedFile.WriteLine(str);

                //write new method code
                while ((str = payloadFile.ReadLine()) != null)
                {

                    modifiedFile.WriteLine(str);
                }

                //look for the ending of code - looking for "} // end of method"
                while (((str = originalFile.ReadLine()) != null) && !str.Contains("} // end of method"))
                {

                    //do nothing (we don't need the original code anymore)
                }

                //now we are at the end of the method code
                //write the } character
                modifiedFile.WriteLine(str);

                //continue the write the rest of the code until we reach EOF
                while ((str = originalFile.ReadLine()) != null)
                {

                    modifiedFile.WriteLine(str);
                }


                originalFile.Close();
                payloadFile.Close();
                modifiedFile.Close();

                File.Delete(Configuration.DisassembledDir + "\\" + disassembledBinaryName);
                File.Move(Configuration.DisassembledDir + "\\" + tempFileName, Configuration.DisassembledDir + "\\" + disassembledBinaryName);
           

            }
            catch (Exception e)
            {
                throw new Exception("Error while replacing with the payload  " + payloadInjectedCode.InjectedCodeFile + "(" + e.Message + ")");
            }

        }

        private void HandleBeginningToMaxStack(StreamReader originalFile, StreamReader payloadFile, StreamWriter fileout, int maxStackSize)
        {
            string currentLine = null;
            //Continue to read until we reach the ".maxstack" line
            while (((currentLine = originalFile.ReadLine()) != null) && !currentLine.Contains(".maxstack"))
            {
                fileout.WriteLine(currentLine);
            }
            //now we are at the ".maxstack" line
            int index = currentLine.IndexOf(".maxstack");
            int num2 = int.Parse(currentLine.Substring(index + ".maxstack".Length));
            //assuming worst case
            currentLine = currentLine.Replace(num2.ToString(), (num2 + maxStackSize).ToString());
            fileout.WriteLine(currentLine); 
        }
        private void MergeCodePost(StreamReader originalFile, StreamReader payloadFile, StreamWriter fileout, int maxStackSize, bool considerLineNumbering)
            //this method is responsible for injecting the payload at the end 
            //of the target method (POST injection)
        {        
            string currentLine = null;
            string retLine;
            int distance = -1;
            try
            {
                //we are just after the { character
                //handle maxstack directive recalculation
               HandleBeginningToMaxStack(originalFile, payloadFile, fileout, maxStackSize);                                                    
                
                //Continue to read until you reach end of the first file
                string nextLine = "";
                while ((currentLine = originalFile.ReadLine()) != null)
                {
                    //Read the next line
                    //is it a ret line 
                    if (CodeUtilities.IsRetLine(currentLine))
                    {
                        nextLine = originalFile.ReadLine();
                        //is it the end of method
                        if (CodeUtilities.IsEndLine(nextLine))
                        {
                            break;
                        }
                        fileout.WriteLine(currentLine);
                        fileout.WriteLine(nextLine);
                    }
                    else
                    {
                        fileout.WriteLine(currentLine);
                    }
                }
                //now we have ret
                //no need for this ret line, it will be replaced
                //write the method line, for later saving
                retLine = currentLine;

                if (considerLineNumbering) 
                      distance = CodeUtilities.toInt(CodeUtilities.ParseLineNumber(CodeUtilities.ExtractLineNumber(currentLine)));
               
                //Continue to read until you reach end of the second file
                while ((currentLine = payloadFile.ReadLine()) != null)
                {
                    //Read the next line
                    if (considerLineNumbering)
                        fileout.WriteLine(CodeUtilities.ReplaceLineNumberInCode(currentLine, distance));
                    else
                        fileout.WriteLine(CodeUtilities.CreateUniqueLabels(currentLine));
                }
                

                fileout.WriteLine(Configuration.ret);

                // read } before, write it
                fileout.WriteLine(nextLine);
            }
            catch (Exception )
            {
                throw new Exception("Error while merging with payload code");
            }
        }
        private void MergeCodePre(StreamReader originalFile, StreamReader payloadFile, StreamWriter fileout, int maxStackSize, bool considerLineNumbering)
        {
            //this method is responsible for injecting the payload at the beginning 
            //of the target method (PRE injection)
            //inject at the first encountered IL code line 
            {
                string currentLine = "", payloadLine = "", payloadLine2 = "",nextLine = "", notEmptyLine="";
                int distance = -1;
                try
                {
                    //we are just after the { character
                    //handle maxstack directive recalculation
                    HandleBeginningToMaxStack(originalFile, payloadFile, fileout, maxStackSize);

                    //read until first line of code (skip variables, etc.)
                    while (((currentLine = originalFile.ReadLine()) != null) &&  !CodeUtilities.IsLineOfCode(currentLine))
                    {
                        fileout.WriteLine(currentLine);
                    }
                    //currentLine contains the first original line of code - used later

                    //read payload code, skip ret
                    while (((payloadLine2= payloadFile.ReadLine()) != null) )
                    {
                        payloadLine = payloadLine2;
                        if (!payloadLine.Equals(""))
                            notEmptyLine = payloadLine;
                        if (considerLineNumbering) 
                            fileout.WriteLine(payloadLine);                        
                        else
                            fileout.WriteLine(CodeUtilities.CreateUniqueLabels(payloadLine));                        
                    }

                    //handle empty lines at end of file
                    if (payloadLine.Equals(""))
                        payloadLine = notEmptyLine;


                    //payloadLine is the last line (used to contain ret), use the line for distance calculation


                    if (considerLineNumbering)
                    {
                         distance = CodeUtilities.toInt(CodeUtilities.ParseLineNumber(CodeUtilities.ExtractLineNumber(payloadLine))) + Configuration.defaultDistance;
                        fileout.WriteLine(CodeUtilities.ReplaceLineNumberInCode(currentLine, distance));
                    }
                    else
                        fileout.WriteLine(currentLine);


                    //Continue to read until you reach end of the original method
                    while ((currentLine = originalFile.ReadLine()) != null)
                    {
                        if (CodeUtilities.IsRetLine(currentLine))
                        {
                            nextLine = originalFile.ReadLine();
                            //is it the end of method
                            if (CodeUtilities.IsEndLine(nextLine))
                            {
                                if (considerLineNumbering)                  
                                    fileout.WriteLine(CodeUtilities.ReplaceLineNumberInCode(currentLine, distance));
                                 else
                                    fileout.WriteLine(currentLine);
                                break;
                            }

                            if (considerLineNumbering)
                            {
                                fileout.WriteLine(CodeUtilities.ReplaceLineNumberInCode(currentLine, distance));
                                fileout.WriteLine(CodeUtilities.ReplaceLineNumberInCode(nextLine, distance));
                            }
                            else
                            {
                                fileout.WriteLine(currentLine);
                                fileout.WriteLine(nextLine);
                            }
                        }
                        else
                        {
                            if (considerLineNumbering)
                                fileout.WriteLine(CodeUtilities.ReplaceLineNumberInCode(currentLine, distance));
                            else
                                fileout.WriteLine(currentLine, distance);
                        }
                    }
                    // read } before, write it
                    fileout.WriteLine(nextLine);
                }
                catch (Exception)
                {
                    throw new Exception("Error while merging with payload code");
                }
            }

        }
    }
}

