using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ReFrameworker
{
    public partial class MainForm : Form
    {
        ParseItemXML currentItem;
        public MainForm()
        {
            InitializeComponent();
            try
            {
                ParseConfigXML.ParseFile("Config.xml");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message);
            }
            SetConfig();
        }

        private void SetConfig()
        {
            copyFromRuntimeToolStripMenuItem.Checked = Configuration.copyFromRuntime;
            promptForDeployerToolStripMenuItem.Checked = Configuration.promptForDeployer;
        }

        private void DisplayProgress(string s)
        {
            progresslistBox.Items.Add(s);
            Update();
        }

        private void ClearDisplay()
        {
            progresslistBox.Items.Clear();
            ReferenceslistBox.Items.Clear();
            MethodslistBox.Items.Clear();
            PayloadslistBox.Items.Clear();
            ClasseslistBox.Items.Clear();
            Update();
        }

        private void loadItemToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ClearDisplay();
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.InitialDirectory = Environment.CurrentDirectory +"\\"+ Configuration.ItemsDir;
            dlg.RestoreDirectory = true;
            dlg.Filter = "Item Files (*.item)|*.item|All files (*.*)|*.*";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    currentItem = new ParseItemXML(dlg.FileName);
                
                itemtextBox.Text = currentItem.GetItemName();
                DescriptiontextBox.Text = currentItem.GetDescription();
                BinaryNametextBox.Text = currentItem.GetBinaryName();
                BinaryLocationtextBox.Text = currentItem.GetBinaryLocation();
                precompiledImagetextBox.Text = currentItem.GetPrecompiledImageLocation();

                foreach (ReferenceModule externalRef in currentItem.GetRefernces())
                    ReferenceslistBox.Items.Add(externalRef.InjectedRefFile);
                foreach (MethodModule payloadMethod in currentItem.GetMethods())
                    MethodslistBox.Items.Add(payloadMethod.MethodCodeFile);
                foreach (ClassModule payloadClass in currentItem.GetClasses())
                    ClasseslistBox.Items.Add(payloadClass.ClassCodeFile);
                foreach (PayloadModule payloadCode in currentItem.GetInjectedPayloads())
                    PayloadslistBox.Items.Add(payloadCode.InjectedCodeFile);

                DisplayProgress("Item "+ currentItem.GetItemName() + " loaded successfully");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            }
        }

        private void AboutReFrameworkerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MessageBox.Show("ReFrameworker was developed by Erez Metula (ErezMetula@AppSec.co.il) as a Framework modification tool", "ReFrameworker " + Configuration.version,
            MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private bool DecideCopy(string filename, string location)
        {
            string fullpath = location + "\\" + filename;
            if (Configuration.copyFromRuntime)
            {
                if (FileUtilities.IsFileExists(Configuration.InputDir + "\\" + filename))
                {
                    if (MessageBox.Show("Overwrite exisiting file?", "Overwrite?", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                    {
                        FileUtilities.CopyFromRuntimeLocation(filename, fullpath);
                    }
                }
                else
                    FileUtilities.CopyFromRuntimeLocation(filename, fullpath);
            }
            else
            {
                if (!FileUtilities.IsFileExists(Configuration.InputDir + "\\" + filename))
                {
                    MessageBox.Show("The file " + filename + " not exists in your Input directory. Please choose \"Extract binary from Runtime\" in order to continue.");
                    return false;
                }
            }
            return true;
        }

        private bool CheckExternalExecutables()
        {
            if (!FileUtilities.IsFileExists(Configuration.assemblerLocation))
                if (MessageBox.Show("ilasm.exe not found. Continue anyway?", "ilasm.exe not found", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No)
                {
                    MessageBox.Show("set the \"assemblerLocation\" section in Config.xml to the correct location of ilasm.exe");
                    return false;
                }

            if (!FileUtilities.IsFileExists(Configuration.disassemblerLocation))
                if (MessageBox.Show("ildasm.exe not found. Continue anyway?", "ildasm.exe not found", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No)
                {
                    MessageBox.Show("set the \"disassemblerLocation\" section in Config.xml to the correct location of ildasm.exe (make sure you have the .NET Framework SDK installed)");
                    return false;
                }

            if (!FileUtilities.IsFileExists(Configuration.nativeCompilerLocation))
                if (MessageBox.Show("ngen.exe not found. Continue anyway?", "ngen.exe not found", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.No)
                {
                    MessageBox.Show("set the \"nativeCompilerLocation\" section in Config.xml to the correct location of ngen.exe");
                    return false;
                }
    
            return true;
        }

        private void startToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (currentItem == null)
            {
                MessageBox.Show("You should load an item first..");
                return;
            }

            if (!CheckExternalExecutables())
            {
                MessageBox.Show("Execution stopped. Modify Config.xml and run again.");
                return;
            }

            try
            {
                CodeChanger changer = new CodeChanger(currentItem.GetBinaryName());
                if (!DecideCopy(currentItem.GetBinaryName(), currentItem.GetBinaryLocation()))
                    return;
                DisplayProgress("Disassemblying the binary " + currentItem.GetBinaryName() + " (it may take a while..)");
                Cursor.Current = Cursors.WaitCursor;
                //disassemble the binary
                changer.Disassemble();
                Cursor.Current = Cursors.Default;
               
                //inject all payloads
                foreach (PayloadModule payloadCode in currentItem.GetInjectedPayloads())
                {
                    DisplayProgress("Injecting Payload from file: " + payloadCode.InjectedCodeFile );

                    switch (payloadCode.InjectionMode)
                    {
                        case PayloadModule.InjectionModeEnum.PreAppend:
                            changer.InjectPayloadCode(payloadCode, payloadCode.InjectionMode);
                            break;
                        case PayloadModule.InjectionModeEnum.PostAppend:
                            changer.InjectPayloadCode(payloadCode, payloadCode.InjectionMode);
                            break;
                        case PayloadModule.InjectionModeEnum.Replace:
                            changer.ReplaceWithPayloadCode(payloadCode);
                            break;
                    }


                }


                
                //inject all methods
                foreach (MethodModule payloadMethod in currentItem.GetMethods())
                {
                    DisplayProgress("Injecting Method from file: " + payloadMethod.MethodCodeFile );
                    changer.InjectMethod(payloadMethod);
                }

                //inject all classes
                foreach (ClassModule payloadClass in currentItem.GetClasses())
                {
                    DisplayProgress("Injecting Class from file: " + payloadClass.ClassCodeFile );
                    changer.InjectClass(payloadClass);
                }

                //inject all references
                foreach (ReferenceModule externalRef in currentItem.GetRefernces())
                {
                    DisplayProgress("Injecting Reference from file: " + externalRef.InjectedRefFile);
                    changer.InjectExternalRef(externalRef);
                }


                DisplayProgress("Assemblying the binary  " + currentItem.GetBinaryName() + " (it may take a while..)");
                Cursor.Current = Cursors.WaitCursor;
                //assemble the modified code
                changer.Assemble();
                Cursor.Current = Cursors.Default;
                MessageBox.Show("Done!! Your modified binary is ready to be deployed...", "Finish");
                DisplayProgress("Done!! Your modified binary is ready to be deployed...");
                if (MessageBox.Show("Generate batch deployer?", "Deployer", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
                {
                    DisplayProgress("Generating deployer/undeployer for " + currentItem.GetBinaryName());
                    FileUtilities.GenerateDeployer(currentItem.GetBinaryName(), currentItem.GetBinaryLocation(), currentItem.GetPrecompiledImageLocation());
                    DisplayProgress("Use " + Configuration.deployFileName + " to inject your modified binary, and " + Configuration.undeployFileName + " to restore the original binary");
                }
                DisplayProgress("Finished successfuly");
            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void copyFromGACToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Configuration.copyFromRuntime = !Configuration.copyFromRuntime;
        }

        private void promptForDeployerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Configuration.promptForDeployer = !Configuration.promptForDeployer;
        }

        private void deleteProcessingDirectoryContentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to delete the content of the directories " + Configuration.InputDir + "," + Configuration.DisassembledDir + "," + Configuration.OutputDir + "?", "Delete directory content", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk, MessageBoxDefaultButton.Button1) == DialogResult.Yes)
            {
                FileUtilities.DeleteDirContent(Configuration.InputDir);
                FileUtilities.DeleteDirContent(Configuration.DisassembledDir);
                FileUtilities.DeleteDirContent(Configuration.OutputDir);
                DisplayProgress("Directory content was cleaned");
            }
        }

       

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void helpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.appsec.co.il");
        }

               
    }    
}