using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;


namespace ReFrameworker
{
    class ParseItemXML
    {
        string description, itemName;
        string binaryName, binaryLocation, precompiledImageLocation;
        string injectionMode;
        ArrayList references, methods, injectedpayloads, classes;
        public ArrayList GetRefernces() { return references; }
        public ArrayList GetMethods() { return methods; }
        public ArrayList GetClasses() { return classes; }
        public ArrayList GetInjectedPayloads() { return injectedpayloads; }
        public string GetBinaryName() { return binaryName; }
        public string GetBinaryLocation() { return binaryLocation; }
        public string GetPrecompiledImageLocation() { return precompiledImageLocation; }
        public string GetDescription() { return description; }
        public string GetItemName() { return itemName; }

        public string GetInjectionMode() { return injectionMode; }

        string FileName, Location;
        int stackSize;
        bool beforeLocation;
        bool considerLineNumbering;

        private void SetDefaults()
        {
            stackSize = Configuration.defaultStackSize;
            injectionMode = Configuration.defaultInjectionMode; //append , replace
            beforeLocation = Configuration.defaultBeforeLocation;
            considerLineNumbering = Configuration.defaultConsiderLineNumbering;
        }


        public ParseItemXML(string XMLfile)
        {
            references = new ArrayList();
            methods = new ArrayList();
            classes = new ArrayList();
            injectedpayloads = new ArrayList();
            itemName = XMLfile.Substring(XMLfile.LastIndexOf('\\') + 1, XMLfile.Length - XMLfile.LastIndexOf('\\') - 1);
            try
            {
                ParseFile(XMLfile);
            }
            catch (Exception ex)
            {
                throw ex;
            }           
        }




        public void ParseFile(string XMLfile)
        {
            SetDefaults();
            FileName = "";
            Location = "";


            try
            {
                XmlTextReader objXmlTextReader = new XmlTextReader(XMLfile);
                string sName = "";
                while (objXmlTextReader.Read())
                {
                    switch (objXmlTextReader.NodeType)
                    {
                        case XmlNodeType.Element:
                            sName = objXmlTextReader.Name;
                            break;
                        case XmlNodeType.CDATA:
                            Location = objXmlTextReader.Value;
                            break;
                        case XmlNodeType.Text:
                            switch (sName)
                            {
                                case "Description":
                                    description = objXmlTextReader.Value;
                                    break;
                                case "BinaryName":
                                    binaryName = objXmlTextReader.Value;
                                    break;
                                case "BinaryLocation":
                                    binaryLocation = objXmlTextReader.Value;
                                    break;
                                case "PrecompiledImageLocation":
                                    precompiledImageLocation = objXmlTextReader.Value;
                                    break;
                                case "FileName":
                                    FileName = objXmlTextReader.Value;
                                    break;
                                case "BeforeLocation":
                                    beforeLocation = objXmlTextReader.Value.ToUpper().Equals("TRUE") ? true : false;
                                    break;
                                case "StackSize":
                                    stackSize = int.Parse(objXmlTextReader.Value);
                                    break;
                                case "ConsiderLineNumbering":
                                    considerLineNumbering = objXmlTextReader.Value.ToUpper().Equals("TRUE") ? true : false;
                                    break;
                                case "InjectionMode":
                                    injectionMode = objXmlTextReader.Value;
                                    break;
                            }
                            break;
                        case XmlNodeType.EndElement:
                            switch (objXmlTextReader.Name)
                            {
                                case "Reference":
                                    references.Add(new ReferenceModule("TBD", FileName));
                                    SetDefaults();
                                    break;
                                case "Method":
                                    methods.Add(new MethodModule("TBD", FileName, Location, beforeLocation));
                                    SetDefaults();
                                    break;
                                case "Class":
                                    classes.Add(new ClassModule("TBD", FileName, Location, beforeLocation));
                                    SetDefaults();
                                    break;
                                case "Payload":
                                    string mode = injectionMode.ToUpper();
                                    PayloadModule.InjectionModeEnum modeEnum;
                                    if (mode.Equals("REPLACE"))
                                        modeEnum = PayloadModule.InjectionModeEnum.Replace;
                                    else
                                        if (mode.Equals("PRE APPEND"))
                                            modeEnum = PayloadModule.InjectionModeEnum.PreAppend;
                                        else if (mode.Equals("POST APPEND"))
                                            modeEnum = PayloadModule.InjectionModeEnum.PostAppend;
                                        else //default is post append mode
                                            modeEnum = PayloadModule.InjectionModeEnum.PostAppend;
                                    injectedpayloads.Add(new PayloadModule("TBD", FileName, Location, stackSize, modeEnum, considerLineNumbering));
                                    SetDefaults();
                                    break;
                            }
                            break;
                    }
                }
            }
            catch (Exception)
            {
                throw new Exception("Error while parsing file " + XMLfile);

            }
        }
    }
}

